<?php

/**
 * Gets details of the latest flight arrival/departures at Male' International 
 * Airport.
 *
 * Usage example:
 * $flights = new flightinfo();
 * $flights->getInfo( 'arrivals', 'all' );
 * $flights->sortBy( 'estimated' );
 * echo $flights->updatetime;
 * print_r( $flights->flights );
 *
 *
 * PHP version 5
 *
 * LICENSE: Freeware. Distribution and modification allowed with credits intact.
 *
 * @author 		Jawish Hameed <jaa@technova.com.mv>
 * @version 	0.2 (2007-11-23) 
 * @link 		http://www.jawish.org/
 */


// {{{ FlightInfo class

/**
 * Processes flight information data file and makes it available as easily
 * accessible properties of an object
 *
 * @package 	flightinfo
 * @author 		Jawish Hameed <jaa@jawish.org>
 * @version 	0.2 (2006-11-23)
 */
class flightinfo
{
	// {{{ properties
	
	/**
	 * Time of last update to flight data
	 * Time is in unix timestamp format
	 *
	 * @var integer
	 * @access public
	 */
	public $updatedtime;
	
	/**
	 * Returns the flights details as a 2 multidimensional array
	 *
	 * Array is of the form Array(DATE => FLIGHT) where DATE is a unix timestamp
	 * FLIGHT is an array with the elements: DATE, AIRLINE NAME, AIRLINE ID, FLIGHT ID, 
	 * ROUTE, SCHEDULED ARRIVAl, ESTIMATED ARRIVAL and STATUS. The ROUTE is presented 
	 * as an indexed array with the route stops in order
	 *
	 * @var array
	 * @access public
	 */
	public $flights;

	// }}}
	
	
	// {{{ class constructor
	
	/**
	 * Initializes object and optionally fetches flight data
	 *
	 * @param string	$type		set to fetch 'arrivals' or 'departures' data
	 *
	 * @access public
	 */
	public function __contruct($type = '')
	{
		// Get info if type parameter specified
		if ($type != '') $this->getInfo($type);
	}
	
	// }}
	
	
	// {{{ getInfo
	
	/**
	 * Gets flight data as specified
	 *
	 * @param string	$type		Set to fetch 'arrivals' or 'departures' data
	 * @return boolean	True if flight is processed successfully, false if not.
	 *
	 * @access public
	 */
	public function getInfo($type = 'arrivals')
	{
		// Check whether to get arrival or departure data
		if ($type == 'arrivals') {
			$link = 'http://www.fis.com.mv/cgi-bin/webfids.pl?webfids_type=arrivals';
		} else {
			$link = 'http://www.fis.com.mv/cgi-bin/webfids.pl?webfids_type=departures';
		}
		
		// Setup regular expression patterns for data extraction
		$patUpdated = '/<td class="updatedtime">UPDATED: (.*)<\/td>/';
		$patDate 	= '/&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;(.*)<\/td>/';
		$patDayEntries = '/DOMESTIC &amp; INTERNATIONAL(.*)(summaryhead|<\/table>)/Us';
		$patEntry 	= '/<tr class="schedulerow.*" valign="top">(.*)<\/tr>/Us';
		
		// Execute main code with exception handling
		try {
			
			// Fetch data from FIS site
			$data = file_get_contents($link);
			
			// Extract update time
			preg_match($patUpdated, $data, $matches);
			$matches[1] = str_replace(array('|', ','), '', $matches[1]);
			$this->updatedtime = strtotime($matches[1]);
			
			// Locate entries
			// Return with failure if no entries can be found
			$entries = array();
			if (!preg_match_all($patDayEntries, $data, $daymatches)) return false;
		
			// Clear unneeded memory
			unset($data);
			
			// Loop through the each day listing
			while ($daymatches[0]) {
				
				// Get next day
				$daymatch = array_shift($daymatches[0]);
				
				// Extract date for listing
				preg_match($patDate, $daymatch, $temp);
				$date = strtotime($temp[1]);
				
				// Extract entries for the day
				preg_match_all($patEntry, $daymatch, $matches);
				
				// Process the extracted entries
				while ($matches[1]) {
					
					// Get next entry
					$match = array_shift($matches[1]);
					 
					// Start off new entry with data element
					$fdata = array();
					
					// Extract the fields
					preg_match_all('/<td class="(.*)".*>(.*)<\/td>/Us', $match, $temp);
					
					// Get airline name and id
					preg_match('/alt="(.*)"/', $temp[2][0], $fdata['airlinename']);
					preg_match('/images\/(.*)\./', $temp[2][0], $fdata['airlineid']);
					$fdata['airlinename'] = $fdata['airlinename'][1];
					$fdata['airlineid'] = strtoupper($fdata['airlineid'][1]);
					
					// Get flightid
					$fdata['flightid'] = $temp[2][1];
					
					// Get route
					$delim = ($type == 'arrivals') ? ' / ' : ' \ ';
					$fdata['route'] = explode($delim, $temp[2][2]);
					
					// Get time
					$fdata['scheduled'] = $temp[2][3];
					
					// Get estimated time
					$fdata['estimated'] = ($temp[2][4] == '&nbsp;') ? '' : $temp[2][4];
					
					// Get status
					$fdata['status'] = ($temp[2][5] == '&nbsp;') ? '' : $temp[2][5];
					$fdata['status'] = strip_tags($fdata['status']);
					
					// Add data to flight listing
					$this->flights[$date][] = $fdata;
				
				}
			}
			
			// Return success
			return true;
		
		} catch (Exception $e) {
			// Error occured:
			
			// Return failure
			return false;
		}
	}
	
	// }}}
	
}

// }}}

?>